package eio

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Eio_utils.Nt_pathSource

Windows path syntax.

  • join dir step appends step to dir using "\\" as the directory separator, unless dir already ends in a separator. After a bare drive (C:) no separator is added, to keep the path drive-relative.
  • split: Volume prefixes (C:, \\server\share, \\?\..., \??\...) are never split.
include Eio.Fs.Pi.PATH
Sourceval split : Eio.Fs.path -> (Eio.Fs.path * string) option

The implementation of Path.split.

The implementation of Path.(/).